import netaddr
import re

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.db import models


class CreateTimeBaseObject(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    active = models.BooleanField(default=True, db_index=True)
    added_by = models.CharField(max_length=64, null=True, blank=True)

    class Meta:
        abstract = True


class InventoryObjectLocation(CreateTimeBaseObject):
    shortname = models.CharField(max_length=32, unique=True)
    fullname = models.CharField(max_length=254)

    def __str__(self):
        return self.fullname or self.shortname

    def __unicode__(self):
        return self.fullname or self.shortname


class L2Network(CreateTimeBaseObject):
    name = models.CharField(max_length=254)
    vlanid = models.IntegerField(max_length=5)


class IPv4Network(CreateTimeBaseObject):
    address = models.CharField(max_length=15)
    netmask = models.CharField(max_length=15)
    gateway = models.CharField(max_length=15)
    l2network = models.ForeignKey(
        L2Network, related_name='ipv4_networks', null=True)
    location = models.ForeignKey(
        InventoryObjectLocation, related_name='ipv4_networks', null=True)

    class Meta:
        unique_together = ('address', 'netmask')
        ordering = ('address',)

    def __str__(self):
        return netaddr.IPNetwork('%s/%s' % (self.address, self.netmask))

    def __unicode__(self):
        return netaddr.IPNetwork('%s/%s' % (self.address, self.netmask))


class InventoryObjectType(CreateTimeBaseObject):
    name = models.CharField(max_length=254, unique=True)

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name


class InventoryObjectLabel(CreateTimeBaseObject):
    TYPE_CHOICES = (
        ('GY', 'Grey'),
        ('BL', 'Blue'),
        ('GN', 'Green'),
        ('LB', 'Lightblue'),
        ('OR', 'Orange'),
        ('RD', 'Red'),
    )

    name = models.CharField(max_length=64, unique=True)
    type = models.CharField(
        max_length=2,
        choices=TYPE_CHOICES,
        default='GY'
    )

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name

    def get_bootstrap_color(self):
        colors = {
            'GY': 'default',
            'BL': 'primary',
            'GN': 'success',
            'LB': 'info',
            'OR': 'warning',
            'RD': 'danger',
        }

        return colors.get(self.type, 'default')

    def link(self):
        return '/inventory.html?label=%s' % self.name


class InventoryObject(CreateTimeBaseObject):
    name = models.CharField(max_length=254, unique=True, db_index=True)
    type = models.ForeignKey(
        InventoryObjectType, related_name='inventory_objects', null=True)
    location = models.ForeignKey(
        InventoryObjectLocation, related_name='inventory_objects', null=True)
    label = models.ManyToManyField(
        InventoryObjectLabel, related_name='inventory_objects')
    l2network = models.ManyToManyField(
        L2Network, related_name='inventory_objects', null=True)
    ipv4network = models.ManyToManyField(
        IPv4Network, related_name='inventory_objects', null=True)
    responsible = models.ManyToManyField(
        User, related_name='inventory_objects', db_index=True)

    class Meta:
        ordering = ('-created',)

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    @property
    def cpu(self):
        try:
            return '%sx%s' % (
                self.attributes.get(name='processorcount').value,
                self.attributes.get(name='processor0').value)
        except ObjectDoesNotExist:
            return None

    def dump(self):
        return {
            'name': self.name,
            'hardware': self.hardware,
        }

    @property
    def hardware(self):
        try:
            return self.attributes.get(name='hardware').value
        except ObjectDoesNotExist:
            return None

    @property
    def ipmi_link(self):
        try:
            return 'http://%s' % self.attributes.get(name='ipmi_host').value
        except ObjectDoesNotExist:
            return None

    @property
    def ipmi_user(self):
        return self.inventory_objects.get(
            context='ipmi', name='user') or None

    @property
    def ipmi_password(self):
        return self.inventory_objects.get(
            context='ipmi', name='password') or None

    @property
    def ipv4address(self):
        try:
            return self.attributes.get(name='ipaddress').value
        except ObjectDoesNotExist:
            return None

    @property
    def ipv4networks(self):
        return [
            '%s/%s' % (x.address, x.netmask) for x in self.ipv4network.all()]

    @property
    def jira_issue(self):
        try:
            return 'http://mirantis.jira.com/browse/%s' % self.attributes.get(
                name='jira_issue').value
        except:
            return None

    @property
    def location_pretty(self):
        return self.location.fullname or self.location.shortname

    @property
    def macaddress(self):
        try:
            return [
                x.value for x in self.attributes.filter(
                    name__contains='macaddress')]
        except ObjectDoesNotExist:
            return []

    @property
    def memory(self):
        try:
            return self.attributes.get(name='memorysize').value
        except ObjectDoesNotExist:
            return None

    @property
    def ssh_link(self):
        try:
            return 'ssh://%s' % self.attributes.get(name='fqdn').value
        except ObjectDoesNotExist:
            return None

    @property
    def link(self):
        return '/object/%s.html' % self.pk

    @property
    def label_string(self):
        return ','.join([x.name for x in self.label.all()])

    @property
    def macaddress_string(self):
        return ','.join(self.macaddress)

    @property
    def responsible_string(self):
        return ','.join([x.username for x in self.responsible.all()])

    @property
    def zabbix_link(self):
        try:
            zabbix_url = 'https://monitoring.infra.mirantis.net'
            return '%s/hostinventories.php?hostid=%s' % (
                zabbix_url,
                self.attributes.get(name='zabbix_host_id').value)
        except ObjectDoesNotExist:
            return None


class InventoryObjectCredential(CreateTimeBaseObject):
    name = models.CharField(max_length=255)
    context = models.CharField(max_length=255)
    value = models.CharField(max_length=255)
    inventory_object = models.ForeignKey(
        InventoryObject, related_name='inventory_objects')

    class Meta:
        unique_together = ('name', 'context', 'inventory_object')

    def __unicode__(self):
        return '%s for %s@%s' % (
            self.name, self.context, self.inventory_object.name)


class InventoryObjectAttribute(CreateTimeBaseObject):
    name = models.CharField(max_length=254)
    value = models.TextField()
    object = models.ForeignKey(InventoryObject, related_name='attributes')

    class Meta:
        unique_together = ('name', 'object')
        ordering = ('name',)

    def __unicode__(self):
        return '%s@%s' % (self.name, self.object.name)

    def __str__(self):
        return '%s@%s' % (self.name, self.object.name)

    def awesome_value(self):
        handlers = [
            'fqdn',
            'puppet_manifests_version',
            'zabbix_host_id',
        ]
        if self.name in handlers:
            if self.name == 'zabbix_host_id':
                return '<a href="%s" target="_blank">%s</a>' % (
                    self.object.zabbix_link,
                    self.value
                )
            if self.name == 'puppet_manifests_version':
                sha1commit = re.search(r'([a-f0-9]{40})', self.value)
                if sha1commit and len(sha1commit.groups()) > 0:
                    return '<a href="%s:%s" target="_blank">%s</a>' % (
                        'https://review.fuel-infra.org/#/q/commit',
                        sha1commit.groups()[0],
                        self.value,
                    )
                return self.value
            if self.name == 'fqdn':
                return '<a href="ssh://%s">%s</a>' % (
                    self.value,
                    self.value,
                )

        return self.value
