import csv
import json
import logging
import netaddr
import re

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.db.models import Count
from django.shortcuts import render_to_response
from django.shortcuts import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.template import RequestContext
from django.views.decorators.csrf import csrf_exempt

from inventory.forms import InventoryObjectLocationForm

from inventory.models import InventoryObject
from inventory.models import InventoryObjectAttribute
from inventory.models import InventoryObjectLabel
from inventory.models import InventoryObjectLocation
from inventory.models import InventoryObjectType
from inventory.models import L2Network
from inventory.models import IPv4Network

from security.decorators import token_auth

logger = logging.getLogger(__name__)


@login_required
def ipv4network(request, networkid):
    ipv4network = IPv4Network.objects.get(pk=networkid)

    return render_to_response('inventory/ipv4network.html', {
        'section_inventory': True,
        'subsection_ipv4networks': True,
        'ipv4network': ipv4network,
    }, context_instance=RequestContext(request))


@login_required
def ipv4networks(request):
    ipv4networks = IPv4Network.objects.all()

    return render_to_response('inventory/ipv4networks.html', {
        'section_inventory': True,
        'subsection_ipv4networks': True,
        'ipv4networks': ipv4networks,
    }, context_instance=RequestContext(request))


@login_required
def l2networks(request):
    l2networks = L2Network.objects.all()

    return render_to_response('inventory/l2networks.html', {
        'section_inventory': True,
        'subsection_l2networks': True,
        'l2networks': l2networks,
    }, context_instance=RequestContext(request))


@login_required
def labels(request):
    labels = InventoryObjectLabel.objects.all()

    return render_to_response('inventory/labels.html', {
        'section_inventory': True,
        'subsection_labels': True,
        'labels': labels,
    }, context_instance=RequestContext(request))


@login_required
def location(request, location):
    location_object = InventoryObjectLocation.objects.get(pk=location)

    if request.method == 'POST':
        location_form = InventoryObjectLocationForm(
            request.POST, instance=location_object)
        location_form.save()

        return redirect('/locations.html')

    location_form = InventoryObjectLocationForm(instance=location_object)
    return render_to_response('inventory/location.html', {
        'location': location_object,
        'location_form': location_form,
        'section_inventory': True,
        'subsection_locations': True,
    }, context_instance=RequestContext(request))


@login_required
def locations(request):
    locations = InventoryObjectLocation.objects.all()

    return render_to_response('inventory/locations.html', {
        'locations': locations,
        'section_inventory': True,
        'subsection_locations': True,
    }, context_instance=RequestContext(request))


@login_required
def objectlist(request):
    label = request.GET.get('label')
    responsible = request.GET.get('responsible')

    get_parameters = request.GET.copy()
    if get_parameters.has_key('page'):
        del get_parameters['page']

    try:
        filters = {}
        if label is not None:
            if label == 'None':
                filters['label'] = None
            else:
                filters['label__name'] = label

        if responsible is not None:
            if responsible == 'None':
                filters['responsible'] = None
            else:
                filters['responsible__username'] = responsible

        object_list = InventoryObject.objects.select_related(
            'location', 'type').filter(**filters)
    except ObjectDoesNotExist:
        object_list = []

    paginator = Paginator(object_list, 20)

    page = request.GET.get('page', 1)
    try:
        objects = paginator.page(page)
    except PageNotAnInteger:
        objects = paginator.page(1)
    except EmptyPage:
        objects = paginator.page(paginator.num_pages)

    return render_to_response('inventory/index.html', {
        'get_parameters': get_parameters,
        'label': label,
        'objects': objects,
        'section_inventory': True,
        'subsection_objects': True,
        'request': request,
    }, context_instance=RequestContext(request))


@login_required
def object(request, object_pk):
    obj = get_object_or_404(InventoryObject, pk=object_pk)
    # FIXME: add proper history
    history = []

    return render_to_response('inventory/object.html', {
        'history': history,
        'object': obj,
        'section_inventory': True,
        'subsection_objects': True,
    }, context_instance=RequestContext(request))


@login_required
def search(request):
    query = request.GET.get('query')
    attributes = InventoryObjectAttribute.objects.filter(
        value__icontains=query)

    return render_to_response('inventory/search.html', {
        'attributes': attributes,
        'section_inventory': True,
        'query': query,
    }, context_instance=RequestContext(request))


@csrf_exempt
@token_auth
def api_import(request):
    if request.method == 'POST':
        object_name = request.POST.get('name')
        object_type = request.POST.get('type')
        location = request.POST.get('location')
        object_labels = request.POST.get('labels')

        logger.info('[%s][TYPE] %s' % (object_name, object_type))

        obj, obj_created = InventoryObject.objects.get_or_create(
            name=object_name)

        status = 200
        if object_type:
            obj_type, obj_type_created = \
                InventoryObjectType.objects.get_or_create(
                    name=object_type)

            obj_type.save()
            obj.type = obj_type
            obj.save()

        if location:
            object_location, object_location_created = \
                InventoryObjectLocation.objects.get_or_create(
                    shortname=location)

            object_location.save()
            obj.location = object_location
            obj.save()
            # FIXME: add history addition
            logger.info('[%s][LOCATION] %s' % (obj.name, location))

        for attribute in request.POST.keys():
            if attribute.startswith('attribute_'):
                attribute_name = re.sub('^attribute_', '', attribute)
                attribute_value = request.POST.get(attribute)

                logger.info('[%s][ATTRIBUTE] [%s] %s = %s' % (
                    obj.name, attribute, attribute_name, attribute_value))

                obj_attribute, obj_attribute_created = \
                    InventoryObjectAttribute.objects.get_or_create(
                        name=attribute_name, object=obj)

                if obj_attribute_created or obj_attribute.value != attribute_value:  # noqa
                    # FIXME: add history addition/update

                    obj_attribute.value = attribute_value
                    obj_attribute.save()

        if object_labels:
            for label in json.loads(object_labels):
                label_obj, label_obj_created = \
                    InventoryObjectLabel.objects.get_or_create(
                        name=label)
                label_obj.save()

                # FIXME: add history addition

                obj.label.add(label_obj)

        if 'attribute_ipaddress' in request.POST.keys() and 'attribute_netmask' in request.POST.keys():  # noqa
            net = netaddr.IPNetwork('%s/%s' % (
                request.POST.get('attribute_ipaddress'),
                request.POST.get('attribute_netmask'),
            ))

            ipv4network, ipv4network_created = \
                IPv4Network.objects.get_or_create(
                    address=net.network,
                    netmask=net.netmask)

            obj.ipv4network.add(ipv4network)

            if ipv4network_created:
                ipv4network.save()

                # FIXME: add history addition

        if obj_created:
            status = 201

            # FIXME: add history addition

            obj.save()

        return HttpResponse(
            content=json.dumps({
                'objectid': obj.pk,
                'object_name': obj.name,
                'status': 'ok',
            }),
            content_type='application/json',
            reason='Ok',
            status=status
        )
    else:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'reason': 'Request method should be POST',
            }),
            content_type='application/json',
            reason='Bad Request',
            status=400
        )


@token_auth
def api_object_list(request):
    location = request.GET.get('location')

    if location:
        try:
            location_obj = InventoryObjectLocation.objects.get(
                shortname=location)
            objects = InventoryObject.objects.filter(
                location=location_obj)
        except ObjectDoesNotExist:
            objects = []
    else:
        objects = InventoryObject.objects.all()

    return HttpResponse(
        content=json.dumps([x.name for x in objects]),
        content_type='application/json',
        reason='Ok',
    )


@token_auth
def api_lookup(request):
    query = request.GET.get('query')
    attributes = InventoryObjectAttribute.objects.filter(
        value__icontains=query)

    return HttpResponse(
        content=json.dumps([{
            'object_name': x.object.name,
            'object_id': x.object.pk,
            'attribute': x.name,
            'value': x.value
        } for x in attributes]),
        content_type='application/json',
        reason='Ok')


@token_auth
def api_object_list_inventory(request):
    label = request.GET.get('label')
    if label:
        objects = InventoryObjectLabel.objects.get(
            name=label).inventory_objects.all()
    else:
        objects = InventoryObject.objects.all()

    return HttpResponse(
        content=json.dumps([
            x.dump() for x in objects]),
        content_type='application/json',
        reason='Ok',
    )


@login_required
def api_suggest(request):
    objects = InventoryObject.objects.filter(
        name__icontains=request.GET.get('query', '')
    )[:request.GET.get('limit', 10)]

    return HttpResponse(json.dumps([x.name for x in objects]))


@login_required
def export_object_inventory(request):
    label = request.GET.get('label')
    responsible = request.GET.get('responsible')

    filters = {}
    if label is not None:
        filters['label'] = InventoryObjectLabel.objects.get(name=label)

    if responsible is not None:
        filters['responsible'] = User.objects.get(username=responsible)

    object_list = InventoryObject.objects.filter(**filters)

    columns = [
        'name',
        'location',
        'hardware',
        'macaddress_string',
        'type',
        'label_string',
        'jira_issue',
        'responsible_string',
    ]

    response = HttpResponse(
        content_type='text/csv',
        reason='Ok',
    )
    writer = csv.writer(response)
    writer.writerow(columns)
    for i in object_list:
        writer.writerow([getattr(i, c) for c in columns])

    return response
